import logging
import os
import random
import string
import imageio
import datetime


from multiprocessing import Pool
from PIL import Image

__LOG_DIR__ = '/var/log/pyrest/'

class TimelapseFactory:
    @staticmethod
    def factory(type):
        if type == "GIF":
            return GifTimeLapse()
        else:
            raise Exception("Cannot create timelapse of type " + type)


class Timelapse(object):

    def __init__(self):
        self.images = list()
        self.tmp_dir = "/tmp/"
        self.setupLogger()

    def setupLogger(self):
        log_path = "%s/%s" % (__LOG_DIR__, 'timelapse.log')
        if not os.path.exists(__LOG_DIR__):
            os.makedirs(__LOG_DIR__)

        self.logging = logging.getLogger("TIMELAPSE")
        self.logging.setLevel(logging.DEBUG)

        fh = logging.FileHandler(log_path)
        fh.setLevel(logging.DEBUG)

        ch = logging.StreamHandler()
        ch.setLevel(logging.ERROR)

        formatter = logging.Formatter('%(asctime)s - %(thread)d - %(levelname)s - %(message)s')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)

        self.logging.addHandler(fh)
        self.logging.addHandler(ch)

    def addWMSImage(self, url):
        self.images.append(url)

    def fetchImages(self):
        p = Pool(5)
        imageList = p.map(fetchImage, self.images)
        p.close()
        p.join()
        return imageList

def fetchImage(url):
    return Image.fromarray(imageio.imread(url))

class GifTimeLapse(Timelapse):

    def create(self, delay=1, loop=0):
        save_dir = os.path.join(self.tmp_dir, ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(10)))
        self.logging.debug("Creating temporaray directory at %s ", save_dir)
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        save_dir = save_dir + "/pygif.gif"
        self.logging.debug("Fetching %s images", str(len(self.images)))
        frames = self.fetchImages()
        self.logging.debug(" Creating gif image with delay of %s and looping set to %s", str(delay), str(loop))
        frames[0].save(os.path.join(save_dir), save_all=True, append_images=frames[1:], duration=delay, loop=loop, optimize=False)
        return save_dir