import os

from flask import request, send_file, after_this_request, Blueprint
from flask_restplus import Namespace, fields, Resource
from pyrest.utils.timelapse.Timelapse import TimelapseFactory



timelapse_api = Namespace('timelapse', description='Operations for creating timelapses')

from pyrest.models.timelapse import timelapse_model


# Base timelapse service
@timelapse_api.route('/')
class TimelapseBaseService(Resource):
    def get(self):
        return "<h1>Welcome to the timelapse API</h1>"


# Timelapse creation service
@timelapse_api.route("/<type>")
@timelapse_api.param('type', 'Format of the timelapse to create')
class TimelapseCreationServie(Resource):
    @timelapse_api.expect(timelapse_model)
    @timelapse_api.doc('Create a timelapse based on the type specified in the url parameter')
    def post(self, type):
        data = request.json
        timelapse = TimelapseFactory.factory(type.upper())

        for image in data["images"]:
            timelapse.addWMSImage(image)

        file_path = timelapse.create(data["settings"]["delay"], 0 if data["settings"]["loop"] == "true" else 1)

        @after_this_request
        def remove_file(response):
            os.remove(file_path)
            os.rmdir(os.path.abspath(os.path.join(file_path, os.pardir)))
            return response
        return send_file(file_path, 'image/gif')
